# Needed? There's nothing here!
class Model(object):
    """
    A :class:`Model` defines the various formulae of a particular problem that
    we're interested in. The methods use a :class:`View`, which is a particular
    aspect of some data source.
    """
registered_models = {}